/*

                                   _JNJ`                        
                               .JNMH`                           
                              JMMF`       `;.                   
                            .NMM)          `MN.                 
                            MMM)            (MML                
                           (MMM`             MMML               
                     M  I  D  N  I  G  H  T    C  o  D  E       
                           (NMMF             MHNH               
                            NMML            .MMM                
                             NMML          .NMH                 
                              4MMNL       .#F                   
                               `4HNNL_   `                      
                                   `"""`                        

                          Copyright (C) 2004-2015
       "Ian (Larry) Latter" <ian dot latter at midnightcode dot org>

    Midnight Code is a registered trademark of Ian Latter.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, and mirrored at the Midnight Code web
    site; as at version 2 of the License only.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    (version 2) along with this program; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA, or see http://midnightcode.org/gplv2.txt

*/

/*

  Hardware;

    JayCar (Australia) for misc electronic components;
      EtherTen:           http://www.jaycar.com.au/p/XC4216
      8K2 1W Resistor:    http://www.jaycar.com.au/p/RR2796
      2K7 1W Resistor:    http://www.jaycar.com.au/p/RR2784

*/

#include <avr/wdt.h>
#include <SoftwareSerial.h>

#define PROJECT_NAME "Freezer"
#define PROJECT_VERSION "2.4"

#define PIN_VOLTAGE A4
#define PIN_SERIALT 11  // connects to hub receive
#define PIN_SERIALR 10  // conflicts with eth port
#define PIN_LED_RED 13

#define NUM_SAMPLES 10
#define R1 8200
#define R2 2800
#define SAMPLE_RATE 10

#define OUTPUT_RATE 1000

SoftwareSerial uplink(PIN_SERIALR, PIN_SERIALT);

float denominator;
int sum = 0;
unsigned char samples = 0;
float voltage = 0.0;

void setup()
{
  delay(250);

  wdt_enable(WDTO_8S);
  pinMode(PIN_LED_RED, OUTPUT);
  denominator = (float)((float)R2 / (float)((float)R1 + (float)R2));
  Serial.begin(9600);
  uplink.begin(9600);
  Serial.println();
  Serial.print("------------------");
  Serial.println("----------------");
  Serial.print("MidnightCode Project: ");
  Serial.println(PROJECT_NAME " v" PROJECT_VERSION);
  Serial.print("------------------");
  Serial.println("----------------");
  delay(50);

  // flush out sensor data
  wdt_reset();
  analogRead(PIN_VOLTAGE);
  Serial.println("Setup done.");
  Serial.println();
}

void loop()
{
  wdt_reset();
  samples = 0;
  sum = 0;
  while(samples < NUM_SAMPLES) {
    digitalWrite(PIN_LED_RED, HIGH);
    sum += analogRead(PIN_VOLTAGE);
    samples++;
    digitalWrite(PIN_LED_RED, LOW);
    delay(SAMPLE_RATE);
  }
  voltage = ((float)sum / (float)NUM_SAMPLES * 5.015) / 1024.0;
  voltage = voltage / denominator;
  digitalWrite(PIN_LED_RED, HIGH);
  Serial.println(voltage);
  uplink.println(voltage);
  digitalWrite(PIN_LED_RED, LOW);
  delay(OUTPUT_RATE - SAMPLE_RATE * NUM_SAMPLES);
}
